"use strict";Object.defineProperty(exports, "__esModule", { value: true });exports.default = createRegexTransformStream;exports.replaceStream = replaceStream;var _stream = require("stream");
var _lib = require("./lib");

/**
                              * Create a Transform stream which will maintain a buffer with data received from a Readable stream and write data when the buffer can be matched against the regex.
                              * It will push the whole match object (or objects when the g flag is used) returned by `/regex/.exec(buffer)`.
                              * @param {RegExp} regex Regular expression to execute
                              * @return {Transform} A transform stream.
                              */
function createRegexTransformStream(regex) {
  let buffer = '';

  const ts = new _stream.Transform({
    transform(chunk, encoding, next) {
      let lastMatch;
      let match;
      buffer += chunk.toString();

      // thx stream-snitch
      // https://github.com/dmotz/stream-snitch/blob/master/index.js#L52
      // eslint-disable-next-line no-cond-assign
      while (match = regex.exec(buffer)) {
        ts.push(match);
        lastMatch = match;
        if (!regex.global) break;
      }
      if (lastMatch) {
        buffer = buffer.slice(lastMatch.index + lastMatch[0].length);
      }
      next();
    },
    objectMode: true });


  return ts;
}

/**
   * @typedef {Object} RegexObject
   * @property {RegExp} re regular expression to match against
   * @property {string} replacement a replacement string
   */


/**
       * Create a replacement stream, which will push data when it's done replacing each incoming chunk.
       * @param {RegexObject|RegexObject[]} regex An object or array of objects
       * @returns {Transform} A transform stream, which writes strings
       */
function replaceStream(regex) {
  const re = Array.isArray(regex) ? regex : [regex];
  const fr = re.filter(_lib.checkRegexObject);

  const ts = new _stream.Transform({
    transform(chunk, _, next) {
      let string = `${chunk}`;
      fr.forEach(({ re, replacement }) => {
        string = string.replace(re, replacement);
      });
      ts.push(string);
      next();
    } });


  return ts;
}